'opengl-basic-functions2.inc 
'7th January 2005 
'Code by Todd Riggins and Fletchie with modifications by Jolly Roger 

DEF bmpInfoHeader:BITMAPINFOHEADER   :' bitmap info header 
DEF bmpDataCHAR:POINTER          :' the texture data 
CONST BITMAP_ID =0x04D42         :' Magic Number in Bitmap Header 
'Texture type constants 
CONST nearestfiltered=0 
CONST linearfiltered=1 
CONST mipmapped=2 
DEF glwin:WINDOW 
DEF fullscreen:INT    :' Fullscreen Flag Set 
DEF hDC:UINT       :' Private GDI Device Context 
DEF hRC:UINT        :' Permanent Rendering Context 



SUB LoadBMP(Filename:STRING),POINTER:'Returns a UCHAR Pointer 
   DEF filePtr:POINTER                :' CFILE - File Handle 
   DEF   bmpFileHeader:BITMAPFILEHEADER       :' bitmap file header 
   DEF bmpImageCHAR:POINTER            :' bitmap image data 
   DEF imageIdx:INT                  :' image index counter 
   DEF tempRGB:CHAR                  :' swap variable 
   imageIdx = 0 

   IF(Filename=""):'Make Sure A Filename Was Given 
      MessageBox(NULL,"Filename = NULL","ERROR",@MB_OK) 
      RETURN NULL:'If Not Return NULL 
   ENDIF 

   ' open filename in "read binary" mode 
   filePtr = _fopen(Filename, "rb") 
   IF (filePtr = NULL) 
      MessageBox(NULL,"filePtr = NULL","ERROR",@MB_OK) 
      RETURN NULL 
   ENDIF 

   ' read the bitmap file header 
   _fread(&bmpFileHeader, len(BITMAPFILEHEADER), 1, #<CFILE>filePtr) 
   ' verify that this is a bitmap by checking for the universal bitmap id 
   IF (bmpFileHeader.bfType <> BITMAP_ID) 
      _fclose(filePtr) 
      MessageBox(NULL,"bmpFileHeader.bfType<>BITMAP_ID","ERROR",@MB_OK) 
      RETURN NULL 
   ENDIF 

   ' read the bitmap information header 
   _fread(bmpInfoHeader, len(BITMAPINFOHEADER), 1, filePtr) 
   ' move file pointer to beginning of bitmap data 
   _fseek(#<CFILE>filePtr, bmpFileHeader.bfOffBits, SEEK_SET) 
   ' allocate enough memory for the bitmap image data 
   bmpImageCHAR = NEW(CHAR,bmpInfoHeader.biSizeImage) 
   ' verify memory allocation 
   IF (bmpImageCHAR=NULL) 
      _fclose(filePtr) 
      MessageBox(NULL,"bmpImageCHAR=NULL","ERROR",@MB_OK) 
      RETURN NULL 
   ENDIF 

   ' read in the bitmap image data 
   _fread(bmpImageCHAR, 1, bmpInfoHeader.biSizeImage, #<CFILE>filePtr) 
   ' make sure bitmap image data was read 
   IF (bmpImageCHAR = NULL) 
      _fclose(filePtr) 
      MessageBox(NULL,"Bitmap image wasn't read!","ERROR",@MB_OK) 
      RETURN NULL 
   ENDIF 
   ' swap the R and B values to get RGB since the bitmap color format is in BGR 
   FOR imageIdx = 0 TO (bmpInfoHeader.biSizeImage-3) STEP 3 
      tempRGB = #<CHAR>bmpImageCHAR[imageIdx] 
      #<CHAR>bmpImageCHAR[imageIdx] = #<CHAR>bmpImageCHAR[imageIdx + 2] 
      #<CHAR>bmpImageCHAR[imageIdx + 2] = tempRGB 
   NEXT imageIdx 

   _fclose(filePtr):'Close The Handle 
   RETURN bmpImageCHAR 
ENDSUB 



SUB LoadGLTexture(filename:STRING,texturetype:INT),INT :'Load Bitmaps And Convert To Textures 
   def texturehandle:int 
   texturehandle=0 
    'Load bitmap image data into memory and get pointer to memory 
   bmpDataCHAR = LoadBMP(filename) 

   if(bmpDataCHAR<>NULL) 
      glGenTextures(1, texturehandle):'Create a texture 
      SELECT texturetype 
          CASE nearestfiltered 
          ' Create Nearest Filtered Texture 
           glBindTexture(GL_TEXTURE_2D, texturehandle) 
           glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST) 
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST) 
          glTexImage2D(GL_TEXTURE_2D, 0, 3, bmpInfoHeader.biWidth,bmpInfoHeader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, #<CHAR>bmpDataCHAR) 
        CASE linearfiltered 
          ' Create Linear Filtered Texture 
          glBindTexture(GL_TEXTURE_2D, texturehandle) 
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR) 
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR) 
          glTexImage2D(GL_TEXTURE_2D, 0, 3, bmpInfoHeader.biWidth, bmpInfoHeader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, #<CHAR>bmpDataCHAR) 
          CASE mipmapped 
          ' Create MipMapped Texture 
          glBindTexture(GL_TEXTURE_2D, texturehandle) 
         glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR) 
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST) 
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, bmpInfoHeader.biWidth, bmpInfoHeader.biHeight, GL_RGB, GL_UNSIGNED_BYTE, #<CHAR>bmpDataCHAR) 
        ENDSELECT 
   endif 
   return texturehandle 
ENDSUB 



SUB ReSizeGLScreen()             :' Resize And Initialize The GL Window 
    DEF l,t,w,h:INT 
    GETCLIENTSIZE glwin,l,t,w,h 
    IF w=0 | h=0 THEN RETURN 
   glViewport(0,0,w,h)                  :' Reset The Current Viewport 
   glMatrixMode(GL_PROJECTION)                     :' Select The Projection Matrix 
   glLoadIdentity()                           :' Reset The Projection Matrix 
   ' Calculate The Aspect Ratio Of The Window 
   gluPerspective(45.0,flt(w)/flt(h),0.1,100.0) 
   glMatrixMode(GL_MODELVIEW)                     :' Select The Modelview Matrix 
   glLoadIdentity()                           :' Reset The Modelview Matrix 
RETURN 
ENDSUB 



SUB CloseGLWindow()                              :' Properly Kill The Window 
   IF (fullscreen)                            :' Are We In Fullscreen Mode? 
      ChangeDisplaySettingsA(NULL,0)               :' If So Switch Back To The Desktop 
      ShowCursor(TRUE)             
   ENDIF 

   if (hRC)                                 :' Do We Have A Rendering Context? 
      if (wglMakeCurrent(NULL,NULL)=FALSE)         :' Are We Able To Release The DC And RC Contexts? 
         MessageBox(0,"Release Of DC And RC Failed.","SHUTDOWN ERROR",@MB_OK | @MB_ICONINFORMATION) 
      endif 

      if (wglDeleteContext(hRC)=FALSE)            :' Are We Able To Delete The RC? 
         MessageBox(0,"Release Rendering Context Failed.","SHUTDOWN ERROR",@MB_OK | @MB_ICONINFORMATION) 
      endif 
      hRC=NULL 
   endif 

   if (hDC)                                  :' Are We Able To Delete The DC? 
      if(ReleaseDC(glwin.hwnd,hDC)=FALSE) 
         MessageBox(0,"Release Device Context Failed.","SHUTDOWN ERROR",@MB_OK | @MB_ICONINFORMATION) 
         hDC=NULL 
      endif 
   endif 

   closewindow glwin 
RETURN 
ENDSUB 



/*   This Code Creates Our OpenGL Window.  Parameters Are:               * 
 *   title         - Title To Appear At The Top Of The Window            * 
 *   width         - Width Of The GL Window Or Fullscreen Mode            * 
 *   height         - Height Of The GL Window Or Fullscreen Mode         * 
 *   bits         - Number Of Bits To Use For Color (8/16/24/32)         * 
 *   fullscreenflag   - Use Fullscreen Mode (1) Or Windowed Mode (0)         */ 
SUB CreateGLWindow(title:STRING,width:int,height:int,bits:int,fullscreenflag:CHAR),CHAR 
   def   dwExStyle:UINT               :' Window Extended Style 
   def   dwStyle:UINT               :' Window Style 
   def dmScreenSettings AS DEVMODE 
   def retval AS INT 
   def   WindowRect:WINRECT            :' Grabs Rectangle Upper Left / Lower Right Values 
    DEF l,t:INT 
   WindowRect.left=0                :' Set Left Value To 0 
   WindowRect.right=width            :' Set Right Value To Requested Width 
   WindowRect.top=0               :' Set Top Value To 0 
   WindowRect.bottom=height         :' Set Bottom Value To Requested Height 

   fullscreen=fullscreenflag         :' Set The Global Fullscreen Flag 

   if fullscreen                              :' Attempt Fullscreen Mode? 
      _memset(&dmScreenSettings,0,len(dmScreenSettings)) 
      dmScreenSettings.dmSize=len(dmScreenSettings)      :' Size Of The Devmode Structure 
      dmScreenSettings.dmPelsWidth   = width            :' Selected Screen Width 
      dmScreenSettings.dmPelsHeight   = height         :' Selected Screen Height 
      dmScreenSettings.dmBitsPerPel   = bits            :' Selected Bits Per Pixel 
      dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT 

      ' Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar. 
      retval=ChangeDisplaySettingsA(dmScreenSettings,CDS_FULLSCREEN) 
      if (retval<>DISP_CHANGE_SUCCESSFUL)      :'If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode. 
         if (MessageBox(NULL,str$(retval)+"|"+STR$(dmScreenSettings.dmSize)+","+STR$(dmScreenSettings.dmPelsWidth)+","+STR$(dmScreenSettings.dmPelsHeight)+","+STR$(dmScreenSettings.dmBitsPerPel)+","+hex$(dmScreenSettings.dmFields)+"- The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",@MB_YESNO|@MB_ICONEXCLAMATION)=@IDYES) 
            fullscreen=FALSE            :'Windowed Mode Selected. 
         else                        :'Pop Up A Message Box Letting User Know The Program Is Closing. 
            MessageBox(NULL,"Program Will Now Close.","ERROR",@MB_OK|@MB_ICONSTOP) 
            return FALSE 
         endif 
      endif 
   endif 

   if fullscreen                                    :' Are We Still In Fullscreen Mode? 
      dwExStyle=WS_EX_APPWINDOW                        :' Window Extended Style 
      dwStyle=WS_POPUP|WS_CLIPSIBLINGS|WS_CLIPCHILDREN      :' Windows Style 
      ShowCursor(FALSE)                              :' Hide Mouse Pointer 
   endif 

   'Create The Window 
    IF fullscreen 
       'Full screen mode 
      AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle)   :' Adjust Window To True Requested Size 
      if(FALSE=OPENWINDOW glWin,0,0,WindowRect.right-WindowRect.left,WindowRect.bottom-WindowRect.top,@NOCAPTION,0,title,&glwinproc) 
        ShowCursor(TRUE):' increment internal display counter 
        MessageBox(NULL,"Program Will Now Close.","OPENWINDOW ERROR",@MB_OK|@MB_ICONSTOP) 
        ShowCursor(FALSE):' decrement internal display counter 
        return FALSE 
       ELSE 
          SetWindowLongA(glWin.hwnd,GWL_EXSTYLE,dwExStyle) 
         SetWindowLongA(glWin.hwnd,GWL_STYLE,dwStyle) 
      endif 
    ELSE 
       'Windowed mode 
       if(FALSE=OPENWINDOW glWin,0,0,width,height,@MINBOX|@MAXBOX|@CAPTION,0,title,&glwinproc) 
        MessageBox(NULL,"Program Will Now Close.","OPENWINDOW ERROR",@MB_OK|@MB_ICONSTOP) 
        return FALSE 
       ENDIF 
    ENDIF 

    IF AttachGLScreen(bits)=FALSE THEN RETURN FALSE 

   RETURN TRUE 
ENDSUB 



SUB CreateChildGLWindow(left:INT,top:INT,width:int,height:int,bits:int,parentwindow:WINDOW),CHAR 
    'Open child window 
    if(FALSE=OPENWINDOW glWin,left,top,width,height,@NOCAPTION,parentwindow,"",&glwinproc) 
      MessageBox(NULL,"Program Will Now Close.","OPENWINDOW ERROR",@MB_OK|@MB_ICONSTOP) 
      return FALSE 
    ENDIF 
    IF AttachGLScreen(bits)=FALSE THEN RETURN FALSE 
   RETURN TRUE 
ENDSUB 



SUB AttachGLScreen(colourdepth:INT),INT 
    def   PixelFormat:UINT            :' Holds The Results After Searching For A Match 
   Def pfd AS PIXELFORMATDESCRIPTOR 
   pfd.nSize=len(PIXELFORMATDESCRIPTOR) 
   pfd.nVersion=1 
   pfd.dwFlags=PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER 
   pfd.iPixelType=PFD_TYPE_RGBA 
   pfd.cColorBits=colourdepth 
   pfd.cRedBits=0 
   pfd.cRedShift=0 
   pfd.cGreenBits=0 
   pfd.cGreenShift=0 
   pfd.cBlueBits=0 
   pfd.cBlueShift=0 
   pfd.cAlphaBits=0 
   pfd.cAlphaShift=0 
   pfd.cAccumBits=0 
   pfd.cAccumRedBits=0 
   pfd.cAccumGreenBits=0 
   pfd.cAccumBlueBits=0 
   pfd.cAccumAlphaBits=0 
   pfd.cDepthBits=16 
   pfd.cStencilBits=0 
   pfd.cAuxBuffers=0 
   pfd.iLayerType=PFD_MAIN_PLANE 
   pfd.bReserved=0 
   pfd.dwLayerMask=0 
   pfd.dwVisibleMask=0 
   pfd.dwDamageMask=0 

   hDC=GetDC(glWin.hwnd) 
   if (hDC=NULL) 
      CloseGLWindow()   :' Reset The Display 
      MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",@MB_OK|@MB_ICONEXCLAMATION) 
      return FALSE 
   endif 

   PixelFormat=ChoosePixelFormat(hDC,pfd) 
   if (PixelFormat=FALSE)   :' Did Windows Find A Matching Pixel Format? 
      CloseGLWindow()      :' Reset The Display 
      MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",@MB_OK|@MB_ICONEXCLAMATION) 
      return FALSE 
   endif 

    
   if(SetPixelFormat(hDC,PixelFormat,pfd)=FALSE)   :' Are We Able To Set The Pixel Format? 
      CloseGLWindow()                        :' Reset The Display 
      MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",@MB_OK|@MB_ICONEXCLAMATION) 
      return FALSE 
   endif 

   hRC=wglCreateContext(hDC) 
   if (hRC=NULL)      :' Are We Able To Get A Rendering Context? 
      CloseGLWindow()   :' Reset The Display 
      MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",@MB_OK|@MB_ICONEXCLAMATION) 
      return FALSE 
   endif 

   if(wglMakeCurrent(hDC,hRC)=FALSE)   :' Try To Activate The Rendering Context 
      CloseGLWindow()               :' Reset The Display 
      MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",@MB_OK|@MB_ICONEXCLAMATION) 
      return FALSE 
   endif 

   ShowWindowA(glWin.hwnd,SW_SHOW)   :'Show The Window 
   SetForegroundWindow(glWin.hwnd)   :'Slightly Higher Priority 
   SetFocusA(glWin.hwnd)         :'Sets Keyboard Focus To The Window 
   ReSizeGLScreen()   :'Set Up Our Perspective GL Screen 

RETURN TRUE 
ENDSUB 

'****************************************************************** 
'Fletchie's SaveBitmap code with minor modifications by Jolly Roger 

/* 
TYPE BITMAPINFOHEADER,1 
   DEF biSize:INT 
   DEF biWidth:INT 
   DEF biHeight:INT 
   DEF biPlanes:WORD 
   DEF biBitCount:WORD 
   DEF biCompression:INT 
   DEF biSizeImage:INT 
   DEF biXPelsPerMeter:INT 
   DEF biYPelsPerMeter:INT 
   DEF biClrUsed:INT 
   DEF biClrImportant:INT 
ENDTYPE 
*/ 
TYPE BITMAPFILEHEADER,1 
   DEF bfType:WORD 
   DEF bfSize:INT 
   DEF bfReserved1:WORD 
   DEF bfReserved2:WORD 
   DEF bfOffBits:INT 
ENDTYPE 

TYPE BITMAP,1 
   DEF bmType:INT 
   DEF bmWidth:INT 
   DEF bmHeight:INT 
   DEF bmWidthCHARs:INT 
   DEF bmPlanes:WORD 
   DEF bmBitsPixel:WORD 
   DEF bmBits:INT 
ENDTYPE 

'declare "user32",GetDC(hWnd:int),int 
declare "user32",GetDesktopWindow(),int 
declare "gdi32",CreateCompatibleDC(HDC:int),int 
declare "gdi32",CreateCompatibleBitmap(HDC:int,width:int,height:int),int 
declare "gdi32",DeleteDC(HDC:int),int 
declare "gdi32",SelectObject(hdc:int,hgdiobj:int),int 
declare "gdi32",BitBlt(hdcDest:int,nXDest:int,nYDest:int,nWidth:int,nHeight:int,hdcSrc:int,nXSrc:int,nYSrc:int,dwRop:int),int 
declare "kernel32",ZeroMemory alias RtlZeroMemory(Destination:pointer,Length:int) 
declare "gdi32",GetDIBits(HDC:int,hbitmap:int,start:int,lines:int,bits:MEMORY,info:pointer,usage:INT),int 
declare "gdi32",DeleteObject(hObject:int),int 
declare "gdi32",GetObjectA(handle:int,size:int,mem:MEMORY),int 




sub Save24bitBitMap(filename:string,win:anytype,opt x=-1:int,opt y=0:int,opt w=0:int,opt h=0:int),int 
'global SaveBitMap 
/*HELP 
t:Misc\% 
i:Saves a bitmap of the client area of a window or desktop. 
s:% 
p:filename,[in] The file to save the bitmap to 
p:win,[in] Either a window var or any other var type/number to save the desktop 
p:x y,[in/opt]  The top left of the area 
p:w h,[in/out] The size of the area 
c:If you omit x,y,w,h the entire client area of the window, or entire desktop will be saved. 
c:If you do specify the size of the saved area, you must use suitable values for x,y,w and h 
c:Images are saved at the standard 96dpi. 
c:Under developement - but seems to work ok. 
c:If you try and save at a higher bpp than your screen, it should save at your given BPP but only at your screens BPP quality. 
r:True if saved successfully. 
HELP*/ 
def hdcWin,hdcComp,hbitmap,hbitmapold:int 
def info:BITMAPINFOHEADER 
def fileheader:BITMAPFILEHEADER 
def lpbits:MEMORY 
def filex:BFILE 
def desktop,numberofpixels,pixelnumber,pixelbrightness:int 

TYPE threebytes 
  DEF red:CHAR 
  DEF green:CHAR 
  DEF blue:CHAR 
ENDTYPE 

DEF colourdata:threebytes 

CONST bitsneeded=24 

   desktop=(typeof(win)<>@typeuser) 

   if x=-1 
      if desktop 
         getscreensize w,h 
      else 
         getclientsize ##<window>win,x,y,w,h 
      endif 
      x=0 
      y=0 
   endif 

   if desktop 
      hdcWin=GetDC(GetDesktopWindow()) 
   else 
      hdcWin=GetHDC(##<window>win) 
   endif        

   hdcComp = CreateCompatibleDC(hdcWin) 

   if hdcComp = 0 
      if desktop=0 
         ReleaseHDC #<window>win,hdcWin 
      endif 
      return 0 
   endif 

   hbitmap = CreateCompatibleBitmap(hdcwin,w,h) 

   if hbitmap = 0 
      if desktop=0 
         ReleaseHDC #<window>win,hdcWin 
      endif 
      DeleteDC(hdcComp) 
      return 0 
   endif 

   hbitmapold = SelectObject(hdcComp,hbitmap) 
   BitBlt(hdcComp,0,0,w,h,hdcWin,x,y,0x00CC0020) 
   SelectObject(hdcComp,hbitmapold) 
   DeleteDC(hdcComp) 
'!! 
   ZeroMemory(info,len(info)) 
   CreateInfoStructure(hbitmap,info,bitsneeded) 

   numberofpixels=info.biSizeImage/3 
   allocmem lpbits,3,numberofpixels 

   info.bibitcount=bitsneeded 
   info.biXPelsPerMeter=3780 
   info.biYPelsPerMeter=3780 

   GetDIBits(hdcWin,hbitmap,0,info.biHeight,lpbits,info,0) 
   if desktop=0 
      ReleaseHDC ##<window>win,hdcWin 
   endif 
   DeleteObject(hbitmap) 

   if (openfile(filex,filename,"W") = 0) 
      fileheader.bfType = 0x4d42 
      fileheader.bfSize = len(fileheader) + info.biSize + info.biSizeImage 
      fileheader.bfOffBits = len(fileheader) + info.biSize 
      fileheader.bfReserved1=0 
      fileheader.bfReserved2=0 
      'Write the bitmap data to file 
      write filex,fileheader 
      write filex,info 
      write filex,lpbits 
      closefile filex 
      freemem lpbits 
      return 1 
   else 
      freemem lpbits 
      return 0 
   endif 

endsub 
'------------------------------------------------------------------------------------------ 

sub CreateInfoStructure(hbitmap:int,info:BITMAPINFOHEADER,bitsneeded:int) 

   def bmp:BITMAP 
   def mem:memory 

   allocmem mem,1,len(bmp) 
   GetObjectA(hbitmap,len(bmp),mem) 
   readmem mem,1,bmp 
   freemem mem 
   info.biSize = len(info) 
   info.biWidth = bmp.bmWidth 
   info.biHeight = bmp.bmHeight 
   info.biPlanes = bmp.bmPlanes 
   info.biBitCount = bmp.bmBitsPixel 
   info.biClrUsed = 0 
   info.biCompression = 0 
   def rs:int 
   rs=info.biWidth*int(bitsneeded/8) 
   if (rs % 4)<>0 
      rs=rs+(4-(rs % 4)) 
   endif 
   info.biSizeImage = rs* info.biHeight 
return 
endsub 
